package ca.mcgill.sel.aspectoptima;

import ca.mcgill.sel.aspectoptima.aspects.checkpointing.CheckpointedClass;
import ca.mcgill.sel.aspectoptima.aspects.deferring.DeferredClass;
import ca.mcgill.sel.aspectoptima.aspects.twophaselocking.TwoPhaseLockedClass;
import ca.mcgill.sel.aspectoptima.aspects.optimisticvalidation.OptimisticallyValidatedClass;

aspect ConfigurationEnforcementAspect {
	// declare runtime errors if xor exclusions are violated
	declare error : execution(* (@CheckpointedClass @DeferredClass *).*(..)) : "The configuration parameter \"update strategy\" of the |\"Recovering\" aspect is an exclusive or! You cannot use both options simultaneously!";
	declare error : execution(* (@TwoPhaseLockedClass @OptimisticallyValidatedClass *).*(..)) : "The configuration parameter \"concurrency control\" of the \"Transaction\" aspect is an exclusive or! You cannot use both options simultaneously!";

	// declare runtime errors if composition rules are violated
	declare error : execution(* (@OptimisticallyValidatedClass @CheckpointedClass *).*(..)) : "The value \"Checkpointing\" for the configuration parameter \"update strategy\" is not allowed in combination with the value \"OptimisticValidation\" for the configuration parameter \"concurrency control\"!";
}
