package ca.mcgill.sel.aspectoptima.aspects.lockable;

import java.util.concurrent.locks.ReadWriteLock;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;

public interface Lockable {
	// public method signatures from structural view
	/** acquires a lock (if not already acquired) and locks it using the specified accessKind */
	void getLock(AccessKind accessKind);

	/** releases the lock with the specified accessKind */
	void releaseLock(AccessKind accessKind);
}

aspect LockableAspect {	
	// attributes & associations from structural view
	private Lock Lockable.myLock = new LockImpl();

	// declared methods without message views
	// empty

	// methods shown in message views
	public void Lockable.getLock(AccessKind accessKind) {
		if (this.myLock == null) {
			// BEGIN implementation refinement
			LockableClass lockableClassAnnotation = this.getClass()
					.getAnnotation(LockableClass.class);
			Class<? extends ReadWriteLock> lockClass = lockableClassAnnotation.value();
			try {
				this.myLock = (Lock) lockClass.newInstance();
			} catch (Exception e) {
				e.printStackTrace();
			}
			// END implementation refinement
		}
		this.myLock.lock(accessKind);
	}
	
	public void Lockable.releaseLock(AccessKind accessKind) {
		/* FIXME throw exception in case of unlock without lock */
		this.myLock.unlock(accessKind);
	}
}
