package ca.mcgill.sel.aspectoptima.aspects.lockable;

import java.util.concurrent.locks.ReadWriteLock;

import ca.mcgill.sel.aspectoptimaomtt.aspects.shared.SharedClass;
import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;

public interface LockableForShared {
	// public method signatures from structural view
	/** acquires a lock (if not already acquired) and locks it using the specified accessKind */
	void getSharedLock(AccessKind accessKind);

	/** releases the lock with the specified accessKind */
	void releaseSharedLock(AccessKind accessKind);
}

aspect LockableForSharedAspect {
	// bind annotations to mandatory instantiation parameters
	declare parents : (@LockableClass @SharedClass *) implements LockableForShared;
	
	// attributes & associations from structural view
	private Lock LockableForShared.mySharedLock = new LockImpl();

	// declared methods without message views
	// empty

	// methods shown in message views
	public void LockableForShared.getSharedLock(AccessKind accessKind) {
		if (this.mySharedLock == null) {
			// BEGIN implementation refinement
			LockableClass lockableClassAnnotation = this.getClass()
					.getAnnotation(LockableClass.class);
			Class<? extends ReadWriteLock> lockClass = lockableClassAnnotation.value();
			try {
				this.mySharedLock = (Lock) lockClass.newInstance();
			} catch (Exception e) {
				e.printStackTrace();
			}
			// END implementation refinement
		}
		this.mySharedLock.lock(accessKind);
	}
	
	public void LockableForShared.releaseLock(AccessKind accessKind) {
		/* FIXME throw exception in case of unlock without lock */
		this.mySharedLock.unlock(accessKind);
	}
}
