package ca.mcgill.sel.aspectoptima.aspects.tracing;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import ca.mcgill.sel.aspectoptima.aspects.context.Context;
import ca.mcgill.sel.aspectoptima.aspects.traceable.Trace;

public interface TracingContext extends Context {
	// public method signatures from structural view
	boolean wasAccessed(Traced t);
	Set<Traced> getAccessed();
	List<Trace> getTraces();
	void removeTraces(Set<Traced> t);
}

aspect TracingContextAspect {
	// attributes & associations from structural view
	private Set<Traced> TracingContext.accessed = new HashSet<Traced>();
	private List<Trace> TracingContext.traces = new ArrayList<Trace>();
	
	// declared methods without message views
	public List<Trace> TracingContext.getTraces() {
		// auto-generated getter implementation
		return this.traces;
	}
	
	public Set<Traced> TracingContext.getAccessed() {
		// auto-generated getter implementation
		return this.accessed;
	}

	// methods shown in message views
	public boolean TracingContext.wasAccessed(Traced t) {
		return this.accessed.contains(t);
	}
	
	public void TracingContext.removeTraces(Set<Traced> t) {
		List<Trace> toBeRemoved = new LinkedList<Trace>();
		for(Trace trace : this.traces) {
			Traced target = (Traced) trace.getTarget();
			boolean remove = t.contains(target);
			if (remove) {
				toBeRemoved.add(trace);
			}
		}
		this.traces.removeAll(toBeRemoved);
		this.accessed.removeAll(t);
	}
	
	void TracingContext.addTrace(Trace trace) {
		/* FIXME throw exception if target !instanceof Traced || currentState != active */
		Traced target = (Traced) trace.getTarget();
		this.accessed.add(target);
		this.traces.add(trace);
	}
}
