package ca.mcgill.sel.aspectoptima.aspects.twophaselocking;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;

import org.aspectj.lang.annotation.AdviceName;
import org.aspectj.lang.reflect.MethodSignature;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.lockable.LockableForTwoPhaseLocking;
import ca.mcgill.sel.aspectoptima.aspects.traceable.Trace;
import ca.mcgill.sel.aspectoptima.aspects.traceable.Traceable;
import ca.mcgill.sel.aspectoptima.aspects.tracing.Traced;
import ca.mcgill.sel.aspectoptima.interfacing.ParticipantPool;

public interface TwoPhaseLocked extends Traced, LockableForTwoPhaseLocking {
	// public method signatures from structural view
	// empty
}

aspect TwoPhaseLockedAspect {
	// bind annotations to mandatory instantiation parameters
	declare parents : @TwoPhaseLockedClass * implements TwoPhaseLocked;

	// attributes & associations from structural view
	// empty
	
	// declared methods without message views
	// empty

	// methods shown in message views
	/** acquires a lock for every locked method if the current participant 
	 *  does not already have a lock of this kind on the accessed object */
	@AdviceName("acquireLock")
	before(TwoPhaseLocked target): execution(@TwoPhaseLockedMethod * *.*(..)) && target(target) {
		TwoPhaseLockingParticipant participant = (TwoPhaseLockingParticipant) TwoPhaseLockingParticipantStatic.getCurrent();
		TwoPhaseLockedContext context = (TwoPhaseLockedContext) participant.getContext();
		Method m = ((MethodSignature) thisJoinPointStaticPart.getSignature()).getMethod();
		AccessKind accessKind = target.getAccessKind(m);
		boolean alreadyGotThisLockKind = false;
		List<Trace> currentTraces = context.getTraces();
		Iterator<Trace> currentTracesIterator = currentTraces.iterator();
		while (currentTracesIterator.hasNext() && !alreadyGotThisLockKind) {
			Trace t = currentTracesIterator.next();
			Traceable traceTarget = t.getTarget();
			if (traceTarget == target) {
				AccessKind traceAccessKind = t.getAccessKind();
				if (traceAccessKind == accessKind) {
					alreadyGotThisLockKind = true;
				}
			}
		}
		if (!alreadyGotThisLockKind) {
			target.getLock(accessKind);
		}
	}
}
