package ca.mcgill.sel.aspectoptima.example.banking;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableClass;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableMethod;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;
import ca.mcgill.sel.aspectoptima.example.banking.exception.AccountNotAvailableException;
import ca.mcgill.sel.aspectoptima.example.banking.exception.BankingException;
import ca.mcgill.sel.aspectoptima.example.banking.exception.InsufficientFoundsException;
import ca.mcgill.sel.aspectoptima.interfacing.AbortTransactionOnThrowing;

@TransactableClass(updateStrategy = UpdateStrategy.DEFERRING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING)
public class Bank implements Serializable {
	private Map<Integer, Account> idToAccountMap = new HashMap<Integer, Account>();

	public Bank() {
		/* needed for java.io.Serializable */
	}

	protected Bank(Collection<Account> accounts) {
		super();
		for (Account account : accounts) {
			this.idToAccountMap.put(account.getId(), account);
		}
	}

	@TransactableMethod(accessKind = AccessKind.UPDATE, updateStrategy = UpdateStrategy.DEFERRING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = false)
	@AbortTransactionOnThrowing(BankingException.class)
	protected void transfer(int sourceId, int destinationId, float amount)
			throws InsufficientFoundsException, AccountNotAvailableException {
		/* this order is unusual but necessary to show that operations get
		 * "undone" on abort */
		Account destinationAccount = getAccountForId(destinationId);
		destinationAccount.deposit(amount);
		Account sourceAccount = getAccountForId(sourceId);
		sourceAccount.withdraw(amount);
	}

	private Account getAccountForId(int id) throws AccountNotAvailableException {
		Account account = idToAccountMap.get(id);
		if (account == null) {
			throw new AccountNotAvailableException();
		}
		return account;
	}
}
