package ca.mcgill.sel.aspectoptima.interfacing;

import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactionParticipant;

/** Facade for the AspectOPTIMA framework providing access to all functionality */
public class AspectOPTIMA {
	
	/**
	 * Begins a transaction using the passed update strategy and concurrency control mechanism.
	 * Allows for nested transactions if the parameter nested is true.
	 * @param updateStrategy "CHECKPOINTING" or "DEFERRING"
	 * @param concurrencyControl "TWOPHASELOCKING" or "OPTIMISTICVALIDATION"
	 * @param nested whether nested child transactions should be allowed or not
	 * @return true if the corresponding transaction was successfully started, false in case of any error
	 */
	@SuppressWarnings("unchecked")
	public static boolean beginTransaction(String updateStrategy, String concurrencyControl, boolean nested) {
		try {
			TransactionParticipant participant = AspectOPTIMAInternal.getTransactionParticipant(updateStrategy, concurrencyControl, nested);
			participant.beginTransaction();
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	/**
	 * Begins a nested transaction as a child of the transaction that is associated to the 
	 * currently active thread using the configuration parameters of the parent transaction.
	 * @return true if the corresponding transaction was successfully started, false in case of any error
	 */
	public static boolean beginChildTransaction() {
		try {
			TransactionParticipant participant = (TransactionParticipant) ParticipantPool.claimParticipant();
			participant.beginTransaction();
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	/**
	 * Commits the transaction that is associated to the currently active thread.
	 * @return true if the current transaction was successfully committed, false in case of any error
	 */
	public static boolean commitTransaction() {
		try {
			TransactionParticipant participant = (TransactionParticipant) ParticipantPool.claimParticipant();
			participant.commitTransaction();
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
	
	/**
	 * Aborts the transaction that is associated to the currently active thread.
	 * @return true if the current transaction was successfully aborted, false in case of any error
	 */
	public static boolean abortTransaction() {
		try {
			TransactionParticipant participant = (TransactionParticipant) ParticipantPool.claimParticipant();
			participant.abortTransaction();
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}
}
