package ca.mcgill.sel.aspectoptima.interfacing;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

/**
 * Marks a class as AutoTransacted for the AspectOPTIMA Framework. <br>
 * The parameter updateStrategy() should provide the update strategy (i.e
 * UpdateStrategy.CHECKPOINTING or UpdateStrategy.DEFERRING). <br>
 * The parameter concurrencyControl() should provide the concurrency control
 * mechanism (i.e ConcurrencyControl.TWOPHASELOCKING or
 * ConcurrencyControl.OPTIMISTICVALIDATION). <br>
 * <br>
 * The syntactically equivalent annotation @TransactableClass gets added
 * automatically. All methods of a class that is marked with this annotation are
 * automatically annotated with @TransactableMethod using the corresponding
 * parameter values.
 */
@Target( { ElementType.TYPE })
@Documented
@Retention(RetentionPolicy.RUNTIME)
public @interface AutoTransactedClass {
	UpdateStrategy updateStrategy();

	ConcurrencyControl concurrencyControl();

	boolean nested();
}
