package ca.mcgill.sel.aspectoptima.interfacing;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.AdviceName;
import org.aspectj.lang.reflect.AdviceSignature;


public aspect LoggingAspect {
	declare precedence : *, LoggingAspect;
	
	int levelOfIndent = 0;
	pointcut anyMethodCallOutsideLogged() : (call(* ca.mcgill.sel.aspectoptima..*+.*(..)) || call(ca.mcgill.sel.aspectoptima..*+.new(..))) && !within(LoggingAspect);
	pointcut anyAdviceExecution() : adviceexecution() && within(ca.mcgill.sel.aspectoptima..*) && !within(LoggingAspect);
	pointcut annotatedAdviceExecution(AdviceName adviceNameAnnotation) : adviceexecution() && !within(LoggingAspect) && @this(adviceNameAnnotation);
	Object around(): anyMethodCallOutsideLogged() || anyAdviceExecution() {
		logBeforeProceed(thisJoinPoint, null);
		Object returnedObject = proceed();
		logAfterProceed(returnedObject, thisJoinPoint, null);
		return returnedObject;
	}
	
	Object around(AdviceName adviceNameAnnotation) : annotatedAdviceExecution(adviceNameAnnotation) {
		logBeforeProceed(thisJoinPoint, adviceNameAnnotation);
		Object returnedObject = proceed(adviceNameAnnotation);
		logAfterProceed(returnedObject, thisJoinPoint, adviceNameAnnotation);
		return returnedObject;
	}
	
	private void logBeforeProceed(JoinPoint joinPoint, AdviceName adviceNameAnnotation) {
		StringBuffer log = new StringBuffer();
		Object target = joinPoint.getTarget();
		log.append(Thread.currentThread().getId() + ": ");
		log.append(getIndent());
		log.append(getSignature(target,joinPoint, adviceNameAnnotation));
		log.append(" {\n");
		System.out.print(log);
		System.out.flush();
		levelOfIndent++;
	}
	
	private void logAfterProceed(Object returnedObject, JoinPoint joinPoint, AdviceName adviceNameAnnotation) {
		levelOfIndent--;
		StringBuffer log = new StringBuffer();
		log.append(Thread.currentThread().getId() + ": ");
		log.append(getIndent());
		log.append("} // ");
		log.append(getSignature(null, joinPoint, adviceNameAnnotation));
		if (returnedObject != null) {
			log.append(" returned " + getSimpleArgName(returnedObject));
		}
		log.append("\n");
		System.out.print(log);
		System.out.flush();
	}
	
	private String getIndent() {
		StringBuffer log = new StringBuffer();
		for (int i = 0; i < levelOfIndent; i++) {
			log.append("   ");
		}
		return log.toString();
	}
	
	private String getSignature(Object target,JoinPoint joinPoint, AdviceName adviceNameAnnotation) {
		Signature signature = joinPoint.getSignature();
		Object[] args = joinPoint.getArgs();
		StringBuffer log = new StringBuffer();
		if (signature instanceof AdviceSignature) {
			log.append(signature.getDeclaringType().getSimpleName());
			log.append("." + signature.getName().split("\\$")[1]);
		}
		else {
			String signatureName = signature.getName();
			if (signatureName.equals("<init>")) {
				log.append(getSimpleArgName(signature.getDeclaringTypeName()));
			}
			else if (target != null) {
				log.append(getSimpleArgName(target));
			}
			else {
				log.append(signature.getDeclaringType().getSimpleName());
			}
			log.append("." + signatureName);
		}
		log.append("(");
		for (Object arg : args) {
			log.append(getSimpleArgName(arg) + ", ");
		}
		if (args.length > 0) {
			int length = log.length();
			log.delete(length - 2, length);
		}
		log.append(")");
		return log.toString();
	}
	
	private String getSimpleArgName(Object arg) {
		String simpleArgName = "";
		if (arg != null) {
			String argName = arg.toString();
			int lastPeriod = argName.lastIndexOf(".");
			if (lastPeriod != -1) {
				int lastParenthesisBeforeLastPeriod = argName.substring(0, lastPeriod).lastIndexOf("(");
				if (lastParenthesisBeforeLastPeriod != -1) {
					simpleArgName += argName.substring(0, lastParenthesisBeforeLastPeriod) + "(";
				}
			}
			simpleArgName += argName.substring(lastPeriod + 1);
			
			simpleArgName = simpleArgName.replaceAll("\\([^\\(\\)]*\\)", "");
			return simpleArgName;
			
		}
		else {
			return "";
		}
	}
}
