package ca.mcgill.sel.aspectoptima.test;

import org.junit.Test;
import static org.junit.Assert.assertEquals;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableClass;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableMethod;
import ca.mcgill.sel.aspectoptima.aspects.recovering.RecoverableClass;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

@TransactableClass(updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING)
public class NestedTransaction2PLAdvancedCheckpointingTest {
	NestedTransaction2PLAdvancedCheckpointingTest child;
	int val = 0;

	@TransactableMethod(accessKind = AccessKind.WRITE, updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = true)
	void setVal(int val) {
		this.val = val;
	}
	
	@TransactableMethod(accessKind = AccessKind.WRITE, updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = true)
	void setValOfChild(int val) {
		child.val = val;
	}
	
	@Test
	public void checkpointingNestedNoAcNestAcCommitAbort() {
		NestedTransaction2PLAdvancedCheckpointingTest t = new NestedTransaction2PLAdvancedCheckpointingTest();
		t.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t.val, 2);
		t.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t.val, 1);
	}
	
	@Test
	public void checkpointingNestedNoAcNestAcCommitCommit() {
		NestedTransaction2PLAdvancedCheckpointingTest t = new NestedTransaction2PLAdvancedCheckpointingTest();
		t.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t.val, 2);
		t.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t.val, 3);
	}
	
	@Test
	public void checkpointingNestedNoAcNestAcAbortCommit() {
		NestedTransaction2PLAdvancedCheckpointingTest t = new NestedTransaction2PLAdvancedCheckpointingTest();
		t.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t.val, 1);
		t.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t.val, 3);
	}
	
	@Test
	public void checkpointingNestedNoAcNestAcAbortAbort() {
		NestedTransaction2PLAdvancedCheckpointingTest t = new NestedTransaction2PLAdvancedCheckpointingTest();
		t.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t.val, 1);
		t.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t.val, 1);
	}
	
	@Test
	public void checkpointingNestedChildNoAcNestAcCommitCommit() {
		NestedTransaction2PLAdvancedCheckpointingTest t1 = new NestedTransaction2PLAdvancedCheckpointingTest();
		NestedTransaction2PLAdvancedCheckpointingTest t2 = new NestedTransaction2PLAdvancedCheckpointingTest();
		t1.child = t2;
		t2.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t2.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t2.val, 2);
		assertEquals(t1.child, t2);
		t1.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t1.val, 3);
		assertEquals(t2.val, 2);
		assertEquals(t1.child, t2);
	}
	
	@Test
	public void checkpointingNestedChildNoAcNestAcAbortCommit() {
		NestedTransaction2PLAdvancedCheckpointingTest t1 = new NestedTransaction2PLAdvancedCheckpointingTest();
		NestedTransaction2PLAdvancedCheckpointingTest t2 = new NestedTransaction2PLAdvancedCheckpointingTest();
		t1.child = t2;
		t2.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t2.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t2.val, 1);
		assertEquals(t1.child, t2);
		t1.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(t1.val, 3);
		assertEquals(t2.val, 1);
		assertEquals(t1.child, t2);
	}
	
	@Test
	public void checkpointingNestedChildNoAcNestAcAbortAbort() {
		NestedTransaction2PLAdvancedCheckpointingTest t1 = new NestedTransaction2PLAdvancedCheckpointingTest();
		NestedTransaction2PLAdvancedCheckpointingTest t2 = new NestedTransaction2PLAdvancedCheckpointingTest();
		t1.child = t2;
		t2.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t2.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t2.val, 1);
		assertEquals(t1.child, t2);
		t1.setVal(3);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t1.val, 0);
		assertEquals(t2.val, 1);
		assertEquals(t1.child.val, 1);
	}
	
	@Test
	public void checkpointingNestedChildAddChildrensResults() {
		NestedTransaction2PLAdvancedCheckpointingTest t1 = new NestedTransaction2PLAdvancedCheckpointingTest();
		NestedTransaction2PLAdvancedCheckpointingTest t2 = new NestedTransaction2PLAdvancedCheckpointingTest();
		t1.child = t2;
		t2.val = 1;
		
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t2.setVal(2);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(t1.val, 0);
		assertEquals(t2.val, 1);
		assertEquals(t1.child.val, 1);
	}
	
/*	 CAN'T WORK WITHOUT SOPHISTICATED DEEP COPYING OF CHECKPOINTS
	 TODO IMPLEMENT SOPHISTICATED DEEP COPYING OF CHECKPOINTS
	@Test
	public void checkpointingNestedChildNoAcNestAcCommitAbort() {
		NestedTransaction2PLAdvancedCheckpointingTest t1 = new NestedTransaction2PLAdvancedCheckpointingTest();
		NestedTransaction2PLAdvancedCheckpointingTest t2 = new NestedTransaction2PLAdvancedCheckpointingTest();
		t1.child = t2;
		t2.val = 1;
		
		ca.mcgill.sel.aspectoptima.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		ca.mcgill.sel.aspectoptima.AspectOPTIMATesting.beginCheckpointed2PLNestedTransaction();
		t2.setVal(2);
		ca.mcgill.sel.aspectoptima.AspectOPTIMATesting.commit();
		assertEquals(t2.val, 2);
		assertEquals(t1.child, t2);
		assertEquals(t1.child.val, 2);
		t1.setVal(3);
		ca.mcgill.sel.aspectoptima.AspectOPTIMATesting.abort();
		assertEquals(t1.val, 0);
		assertEquals(t2.val, 1);
		assertEquals(t1.child.val, 1);
	}
*/
}