package ca.mcgill.sel.aspectoptima.test;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableClass;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableMethod;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

@TransactableClass(updateStrategy = UpdateStrategy.DEFERRING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING)
public class NestedTransaction2PLTestDeferring {
	int insideTest = 0;
	
	@TransactableMethod(accessKind = AccessKind.READ, updateStrategy = UpdateStrategy.DEFERRING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = true)
	void foo2(int a) {
		insideTest = a;
		bar2(2, 3);
	}

	@TransactableMethod(accessKind = AccessKind.READ, updateStrategy = UpdateStrategy.DEFERRING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = true)
	void bar2(int a, int b) {
		insideTest = a;
	}

	@Test
	public void deferredBeginBeginAbortCommitTest() {
		NestedTransaction2PLTestDeferring test = new NestedTransaction2PLTestDeferring();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.foo2(1);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.bar2(4, 5);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(2, test.insideTest);
	}

	@Test
	public void deferredBeginBeginAbortAbortTest() {
		NestedTransaction2PLTestDeferring test = new NestedTransaction2PLTestDeferring();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.foo2(1);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.bar2(4, 5);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(0, test.insideTest);
	}

	@Test
	public void deferredBeginBeginCommitAbortTest() {
		NestedTransaction2PLTestDeferring test = new NestedTransaction2PLTestDeferring();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.foo2(1);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.bar2(4, 5);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(0, test.insideTest);
	}

	@Test
	public void deferredBeginBeginCommitCommitTest() {
		NestedTransaction2PLTestDeferring test = new NestedTransaction2PLTestDeferring();
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.foo2(1);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.beginDeferred2PLNestedTransaction();
		test.bar2(4, 5);
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(0, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(4, test.insideTest);
	}
}
