package ca.mcgill.sel.aspectoptima.test;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableClass;
import ca.mcgill.sel.aspectoptima.aspects.transaction.TransactableMethod;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

@TransactableClass(updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING)
public class Transaction2PLTest {
	int insideTest = 0;

	@Test
	public void transactionCheckpointed2PLAbortTest() {
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting
				.beginCheckpointed2PLTransaction();
		Transaction2PLTest test = new Transaction2PLTest();
		test.foo(1);
		test.bar(4, 5);
		assertEquals(4, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.abort();
		assertEquals(0, test.insideTest);
	}

	@TransactableMethod(accessKind = AccessKind.READ, updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = false)
	void foo(int a) {
		insideTest = a;
		bar(2, 3);
	}

	@TransactableMethod(accessKind = AccessKind.READ, updateStrategy = UpdateStrategy.CHECKPOINTING, concurrencyControl = ConcurrencyControl.TWOPHASELOCKING, nested = false)
	void bar(int a, int b) {
		insideTest = a;
	}

	@Test
	public void transactionCheckpoined2PLCommitTest() {
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting
				.beginCheckpointed2PLTransaction();
		Transaction2PLTest test = new Transaction2PLTest();
		test.foo(1);
		test.bar(4, 5);
		assertEquals(4, test.insideTest);
		ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting.commit();
		assertEquals(4, test.insideTest);
	}

	@Test
	public void transactionCheckpointed2PLAutoTest() {
		Transaction2PLTest test = new Transaction2PLTest();
		test.foo(1);
		assertEquals(2, test.insideTest);
		test.bar(4, 5);
		assertEquals(4, test.insideTest);
	}
}
