package ca.mcgill.sel.aspectoptimaomtt.aspects.openmultithreadedtransaction;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

/**
 * Marks a class as Transactable for OpenMultithreadedTransactions within 
 * the AspectOPTIMA Framework. <br>
 * The parameter updateStrategy() should provide the update strategy (i.e
 * UpdateStrategy.CHECKPOINTING or UpdateStrategy.DEFERRING). <br>
 * The parameter concurrencyControl() should provide the concurrency control
 * mechanism (i.e ConcurrencyControl.TWOPHASELOCKING or
 * ConcurrencyControl.OPTIMISTICVALIDATION). <br>
 * <br>
 * (The corresponding internal annotation @TransactableClass
 * will be added automatically.)
 */
@Target( { ElementType.TYPE })
@Documented
@Retention(RetentionPolicy.RUNTIME)
public @interface OMTTransactableClass {
	UpdateStrategy updateStrategy();

	ConcurrencyControl concurrencyControl();
}