package ca.mcgill.sel.aspectoptimaomtt.aspects.openmultithreadedtransaction;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import ca.mcgill.sel.aspectoptima.aspects.accessclassified.AccessKind;
import ca.mcgill.sel.aspectoptima.aspects.transaction.ConcurrencyControl;
import ca.mcgill.sel.aspectoptima.aspects.recovering.UpdateStrategy;

/**
 * Marks a method as Transactable for OpenMultithreadedTransactions within 
 * the AspectOPTIMA Framework.<br>
 * The parameter accessKind() should provide the kind of access (i.e.
 * AccessKind.READ, AccessKind.WRITE, AccessKind.UPDATE) and defaults to
 * AccessKind.WRITE.<br>
 * The parameter updateStrategy() should provide the update strategy (i.e
 * UpdateStrategy.CHECKPOINTING or UpdateStrategy.DEFERRING). <br>
 * The parameter concurrencyControl() should provide the concurrency control
 * mechanism (i.e ConcurrencyControl.TWOPHASELOCKING or
 * ConcurrencyControl.OPTIMISTICVALIDATION). <br>
 * The parameter nested() decides whether nested transactions are allowed. <br>
 * Classes containing OMTTransactableClass methods should be annotated with
 * @OMTTransactableClass. <br><br>
 * (The syntactically equivalent internal annotation @TransactableMethod is added
 * automatically.)
 */
@Target( { ElementType.METHOD })
@Documented
@Retention(RetentionPolicy.RUNTIME)
public @interface OMTTransactableMethod {
	AccessKind accessKind();

	UpdateStrategy updateStrategy();

	ConcurrencyControl concurrencyControl();

	boolean nested();
}