package ca.mcgill.sel.aspectoptimaomtt.aspects.outcomevotable;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;

import ca.mcgill.sel.aspectoptimaomtt.aspects.collaborative.CollaborativeParticipant;
import ca.mcgill.sel.aspectoptimaomtt.aspects.terminatable.TerminatableContext;
import ca.mcgill.sel.aspectoptima.aspects.outcomeaware.BinaryOutcomeImpl;
import ca.mcgill.sel.aspectoptima.aspects.outcomeaware.Outcome;
import ca.mcgill.sel.aspectoptima.aspects.outcomeaware.OutcomeAwareContext;

public interface OutcomeVotableContext extends OutcomeAwareContext, TerminatableContext {
	// public method signatures from structural view
	void registerVote(Outcome v, OutcomeVotableParticipant p);
	boolean alreadyVoted(OutcomeVotableParticipant p);
}

aspect OutcomeVotableContextAspect {
	// attributes & associations from structural view
	private Map<OutcomeVotableParticipant, Outcome> OutcomeVotableContext.votes = new HashMap<OutcomeVotableParticipant, Outcome>();
	
	// declared methods without message views
	private boolean OutcomeVotableContext.isOutcomeDecided() {
		// BEGIN implementation refinement
		Set<CollaborativeParticipant> participants = this.getParticipants();
		return this.votes.keySet().containsAll(participants);
		// END implementation refinement
	}

	// methods shown in message views
	public void OutcomeVotableContext.registerVote(Outcome v, OutcomeVotableParticipant p) {
		this.votes.put(p, v);
		boolean decided = this.isOutcomeDecided();
		if (decided) {
			Outcome outcome = this.computeOutcome();
			this.setOutcome(outcome);
			this.terminateContext(p);
		}
	}
	
	public boolean OutcomeVotableContext.alreadyVoted(OutcomeVotableParticipant p) {
		return this.votes.containsKey(p);
	}
	
	private Outcome OutcomeVotableContext.computeOutcome() {
		Outcome outcome = new BinaryOutcomeImpl(true);
		for (Outcome v : this.votes.values()) {
			boolean positive = v.isPositive();
			if (!positive) {
				outcome = new BinaryOutcomeImpl(false);
			}
		}
		return outcome;
	}
}