package ca.mcgill.sel.aspectoptimaomtt.example.travelagency;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;

import ca.mcgill.sel.aspectoptimaomtt.interfacing.AspectOPTIMAOMTT;
import ca.mcgill.sel.aspectoptimaomtt.interfacing.OMTTThread;

public class TravelAgency {
	FlightBroker flightBroker = new FlightBroker();
	
	public void bookTravel(final String departure, final String destination, final Calendar departureDate) {
		Runnable bookTravelRunnable = new Runnable() { 
			public void run() {
				try {
					AspectOPTIMAOMTT.beginTransactionInCurrentThread("DEFERRING", "TWOPHASELOCKING" ,true);
					final List<Offer> offerList = new ArrayList<Offer>();
					int routePartCount = 0;
					Future<Integer> splitRouteFuture = 
						new FutureTask<Integer>(
							new Callable<Integer>() {
								public Integer call() {
							List<Route> routeList = TravelRouter.getRouteList(departure, destination, departureDate);
							int localRoutePartCount = routeList.size();
							for (final Route r : routeList) {
								Runnable findRoutePartRunnable = new Runnable() {
									public void run() {
										if (r.vehicleType.isPlane()) {
											flightBroker.findAndAddOffer(offerList, r);
										}
									}
								};
								AspectOPTIMAOMTT.spawnAndRunTransactionParticipantFromCurrentThread(findRoutePartRunnable);
							}
							return localRoutePartCount;
							}
						}
					);
					AspectOPTIMAOMTT.spawnAndRunTransactionParticipantFromCurrentThread((Runnable) splitRouteFuture);
					routePartCount = splitRouteFuture.get();
					while (offerList.size() < routePartCount) {
						// wait until all offers are found
						Thread.sleep(1000);
					}
					System.out.println("We found the following travel offers for you.");
					for (Offer offer : offerList) {
						System.out.println(offer);
					}
					System.out.println("Do you want to confirm these offers and book your travel? \n" +
					"Enter \"yes\" to book. \"no\" to cancel, and \"help\" to get assistance from a representative.");
					BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
					String input = in.readLine();
					if (input.toUpperCase().equals("YES")) {
						AspectOPTIMAOMTT.commitTransactionOfCurrentAndSpawnedThreads();
					}
					else if (input.toUpperCase().equals("HELP")) {
						OMTTThread assistanceThread = getAssistanceThread();
						AspectOPTIMAOMTT.joinCurrentTransactionWithExistingThread(assistanceThread);
					}
					else {
						AspectOPTIMAOMTT.abortTransactionOfCurrentThread();
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		};
		AspectOPTIMAOMTT.run(bookTravelRunnable);
	}
	
	OMTTThread getAssistanceThread() {
		/* here we would get the thread of a free service center employee */
		return null;
	}
}

