package ca.mcgill.sel.aspectoptimaomtt.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotSame;

import org.junit.Test;

import ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting;

public class EntrySynchronizingTest {
	static final long timeToSleep = 5000;
	static final long timeToWait = timeToSleep + 1000;
	
	@Test
	public void test() { 
		try {
			Object c = AspectOPTIMAOMTTTesting.beginWithoutEnterEntrySynchronizing();
			AspectOPTIMAOMTTTesting.setMinParticipantCount(2, c);
			EntrySynchronizingRunnable r = new EntrySynchronizingRunnable(c);
			Thread t = new Thread(r);
			t.start();

			Thread.sleep(timeToWait);
			
			EntrySynchronizingRunnable r2 = new EntrySynchronizingRunnable(c);
			Thread t2 = new Thread(r2);
			t2.start();
			
			t.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r.newContext " + r.newContext);
			assertEquals(c, r.newContext);
			
			t2.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r2.newContext " + r2.newContext);
			assertEquals(c, r2.newContext);
			System.out.println("r.newPartcipant " + r.newPartcipant);
			System.out.println("r2.newPartcipant " + r2.newPartcipant);
			assertNotSame(r.newPartcipant, r2.newPartcipant);
			
			long waiting = r.entered - r.triedToEnter;
			System.out.println("waiting = r.entered - r.triedToEnter " + waiting + " = " + r.entered + " - " + r.triedToEnter);
			assertEquals(true, waiting > timeToWait - timeToSleep);
			
			long waiting2 = r2.entered - r2.triedToEnter;
			System.out.println("waiting2 = r2.entered - r2.triedToEnter " + waiting + " = " + r2.entered + " - " + r2.triedToEnter);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}

class EntrySynchronizingRunnable implements Runnable {
	Object context;
	Object newContext;
	Object newPartcipant;
	long triedToEnter;
	long entered;
	
	public EntrySynchronizingRunnable(Object context) {
		this.context = context;
	}

	@Override
	public void run() {
		triedToEnter = System.currentTimeMillis();
		AspectOPTIMAOMTTTesting.joinEntrySynchronizing(context);
		entered = System.currentTimeMillis();
		newContext = AspectOPTIMAOMTTTesting.getActualContext();
		newPartcipant = AspectOPTIMAOMTTTesting.getActualParticipant();
	}
}