package ca.mcgill.sel.aspectoptimaomtt.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotSame;

import org.junit.Test;

import ca.mcgill.sel.aspectoptima.test.AspectOPTIMATesting;

public class OutcomeVotingTest {
	@Test
	public void testCommitAbort() {
		AspectOPTIMAOMTTTesting.beginOutcomeVoting();
		/* let one participant join and vote commit now */
		Object c = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		OutcomeVotingRunnableCommit r = new OutcomeVotingRunnableCommit(c);
		Thread t = new Thread(r);
		t.start();
		/* wait for this thread to die */
		try {
			t.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r.newContext " + r.newContext);
			assertEquals(c, r.newContext);
			System.out.println("AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant() " + AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant());
			System.out.println("r.newPartcipant " + r.newPartcipant);
			assertNotSame(AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant(), r.newPartcipant);
			
			AspectOPTIMATesting.abort();
			assertEquals(false, AspectOPTIMAOMTTTesting.getOutcome(c));
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	@Test
	public void testAbortCommit() {
		AspectOPTIMAOMTTTesting.beginOutcomeVoting();
		/* let one participant join and vote commit now */
		Object c = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		OutcomeVotingRunnableAbort r = new OutcomeVotingRunnableAbort(c);
		Thread t = new Thread(r);
		t.start();
		/* wait for this thread to die */
		try {
			t.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r.newContext " + r.newContext);
			assertEquals(c, r.newContext);
			System.out.println("AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant() " + AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant());
			System.out.println("r.newPartcipant " + r.newPartcipant);
			assertNotSame(AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant(), r.newPartcipant);
			
			AspectOPTIMATesting.commit();
			assertEquals(false, AspectOPTIMAOMTTTesting.getOutcome(c));
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	@Test
	public void testCommitCommit() {
		AspectOPTIMAOMTTTesting.beginOutcomeVoting();
		/* let one participant join and vote commit now */
		Object c = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		OutcomeVotingRunnableCommit r = new OutcomeVotingRunnableCommit(c);
		Thread t = new Thread(r);
		t.start();
		/* wait for this thread to die */
		try {
			t.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r.newContext " + r.newContext);
			assertEquals(c, r.newContext);
			System.out.println("AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant() " + AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant());
			System.out.println("r.newPartcipant " + r.newPartcipant);
			assertNotSame(AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant(), r.newPartcipant);
			
			AspectOPTIMATesting.commit();
			assertEquals(true, AspectOPTIMAOMTTTesting.getOutcome(c));
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	@Test
	public void testCommitCommitAbort() {
		AspectOPTIMAOMTTTesting.beginOutcomeVoting();
		/* let one participant join and vote commit now */
		Object c = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		OutcomeVotingRunnableCommit r = new OutcomeVotingRunnableCommit(c);
		Thread t = new Thread(r);
		t.start();
		/* wait for this thread to die */
		try {
			t.join();
			System.out.println("c " + c);
			assertNotSame(null, c);
			System.out.println("r.newContext " + r.newContext);
			assertEquals(c, r.newContext);
			System.out.println("AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant() " + AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant());
			System.out.println("r.newPartcipant " + r.newPartcipant);
			assertNotSame(AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant(), r.newPartcipant);
			
			OutcomeVotingRunnableCommit r2 = new OutcomeVotingRunnableCommit(c);
			Thread t2 = new Thread(r2);
			t2.start();
			/* wait for this thread to die */
			try {
				t2.join();
				System.out.println("c " + c);
				assertNotSame(null, c);
				System.out.println("r2.newContext " + r2.newContext);
				assertEquals(c, r2.newContext);
				System.out.println("AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant() " + AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant());
				System.out.println("r2.newPartcipant " + r2.newPartcipant);
				assertNotSame(AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant(), r2.newPartcipant);
			
				AspectOPTIMATesting.abort();
				assertEquals(false, AspectOPTIMAOMTTTesting.getOutcome(c));
			
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}

class OutcomeVotingRunnableCommit implements Runnable {
	Object context;
	Object newContext;
	Object newPartcipant;
	
	
	public OutcomeVotingRunnableCommit(Object context) {
		this.context = context;
	}

	@Override
	public void run() {
		AspectOPTIMAOMTTTesting.joinOutcomeVoting(context);
		newContext = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		newPartcipant = AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant();
		AspectOPTIMATesting.commit();
	}
}

class OutcomeVotingRunnableAbort implements Runnable {
	Object context;
	Object newContext;
	Object newPartcipant;
	
	
	public OutcomeVotingRunnableAbort(Object context) {
		this.context = context;
	}

	@Override
	public void run() {
		AspectOPTIMAOMTTTesting.joinOutcomeVoting(context);
		newContext = AspectOPTIMAOMTTTesting.getActualCollaborativeContext();
		newPartcipant = AspectOPTIMAOMTTTesting.getActualCollaborativeParticipant();
		AspectOPTIMATesting.abort();
	}
}